/*
Copyright 2004 John Tsiombikas <nuclear@siggraph.org>

This file is part of the X-Ray ray-tracing library.

X-Ray is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

X-Ray is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with X-Ray; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifndef _LIGHT_HPP_
#define _LIGHT_HPP_

#include <string>
#include "n3dmath2/n3dmath2.hpp"
#include "gfx/3dgeom.hpp"
#include "gfx/color.hpp"

namespace xray {

	// abstract base class Light
	class Light : public XFormNode {
	protected:
		Color color;
		scalar_t intensity;
		scalar_t attenuation[3];

	public:
		Light();
		virtual ~Light();

		virtual void SetColor(const Color &c);
		virtual Color GetColor() const;
	
		virtual void SetIntensity(scalar_t intensity);
		virtual scalar_t GetIntensity() const;
	
		virtual void SetAttenuation(scalar_t att0, scalar_t att1, scalar_t att2);
		virtual scalar_t GetAttenuation(int which) const;
		virtual Vector3 GetAttenuationVector() const;
	};


	class PointLight : public Light {
	public:
		PointLight(const Vector3 &pos=Vector3(0,0,0), const Color &col=Color(1.0f, 1.0f, 1.0f));
		virtual ~PointLight();
	};

	class AreaLight : public Light {
	protected:
		scalar_t radius;
		int shadow_samples;

	public:
		AreaLight(const Vector3 &pos=Vector3(0,0,0), scalar_t rad=1.0, const Color &col=Color(1.0, 1.0, 1.0));
		virtual ~AreaLight();

		virtual void SetRadius(scalar_t rad);
		virtual scalar_t GetRadius() const;

		virtual void SetShadowSamples(int samples);
		virtual int GetShadowSamples() const;
	};
}
	

#endif	// _LIGHT_HPP_
