/*
Copyright 2004 John Tsiombikas <nuclear@siggraph.org>

This file is part of the X-Ray ray-tracing library.

X-Ray is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

X-Ray is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with X-Ray; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <cstdio>
#include "loader.hpp"
#include "common/err_msg.h"

#ifdef _MSC_VER
#define __func__	__FUNCTION__
#endif	// _MSC_VER

using namespace xray;

Scene *xray::LoadScene(const char *fname) {
	
	// read the first 4 bytes to determine the format
	FILE *fp = fopen(fname, "rb");
	if(!fp) return 0;

	char magic[4];
	for(int i=0; i<4; i++) {
		int c;
		if((c = fgetc(fp)) == EOF) {
			warning("(%s): invalid scene file %s", __func__, fname);
			fclose(fp);
			return 0;
		}
		magic[i] = c;
	}
	
	fclose(fp);

	if(magic[0] == '<' && magic[1] == '?' && magic[2] == 'x' && magic[3] == 'm') {
		return LoadXML(fname);
	}
	
	error("only xml scene loading available at the moment");
	return 0;
}
