#include "material.hpp"

using namespace xray;

MatAttr::MatAttr() {
	SetValue(0);
	map = 0;
}

MatAttr::MatAttr(scalar_t val, PixelBuffer *map) {
	SetValue(val);
	SetMap(map);
}

MatAttr::MatAttr(const Vector4 &val, PixelBuffer *map) {
	SetVector(val);
	SetMap(map);
}

void MatAttr::SetValue(scalar_t val) {
	this->val = Vector4(val, val, val, val);
}

void MatAttr::SetVector(const Vector4 &val) {
	this->val = val;
}

void MatAttr::SetMap(PixelBuffer *map) {
	this->map = map;
}

scalar_t MatAttr::GetValue() const {
	return val.x;
}

Vector4 MatAttr::GetVector() const {
	return val;
}

PixelBuffer *MatAttr::GetMap() const {
	return map;
}

/* TODO list:
 * - implement texture filtering
 * - resolve endianess issue
 */
Vector4 MatAttr::operator ()(const TexCoord &tcoord, MatAttrMode mode) const {
	Vector4 res = val;
	
	if(map) {
		int x = (int)(tcoord.u * (scalar_t)map->width) % map->width;
		int y = (int)(tcoord.v * (scalar_t)map->height) % map->height;

		Pixel pcol = map->buffer[y * map->width + x];

		int r = (pcol >> 16) & 0xff;
		int g = (pcol >> 8) & 0xff;
		int b = pcol & 0xff;
		int a = (pcol >> 24) & 0xff;
		Color texel(r, g, b, a);

		if(mode == ATTR_MODE_COLOR) {
			Color tmp = texel * Color(val);
			res = Vector4(tmp.r, tmp.g, tmp.b, tmp.a);
		} else {
			res = Vector4(texel.r * val.x, texel.g * val.y, texel.b * val.z, texel.a * val.w);
		}
	}
	
	return res;
}

Material::~Material() {
	delete [] attr;
}

void Material::SetDefaults() {
	SetAttr(MAT_IOR, MatAttr(1.0));
}

void Material::SetAttr(int attr_id, const MatAttr &attr) {
	this->attr[attr_id] = attr;
}

const MatAttr &Material::GetAttr(int attr_id) const {
	return attr[attr_id];
}

MatAttr *Material::GetAttrPtr(int attr_id) const {
	return &attr[attr_id];
}
