#ifndef _MATERIAL_HPP_
#define _MATERIAL_HPP_

#include "n3dmath2/n3dmath2.hpp"
#include "gfx/3dgeom.hpp"
#include "gfx/pbuffer.hpp"

namespace xray {

	enum MatAttrMode {ATTR_MODE_COLOR, ATTR_MODE_VECTOR};

	class MatAttr {
	private:
		Vector4 val;
		PixelBuffer *map;

	public:
		MatAttr();
		MatAttr(scalar_t val, PixelBuffer *map = 0);
		MatAttr(const Vector4 &val, PixelBuffer *map = 0);
		MatAttr(const Color &col, PixelBuffer *map = 0);
		
		void SetValue(scalar_t val);
		void SetVector(const Vector4 &val);
		void SetColor(const Color &col);
		void SetMap(PixelBuffer *map);

		scalar_t GetValue() const;
		Vector4 GetVector() const;
		Color GetColor() const;
		PixelBuffer *GetMap() const;

		Vector4 operator ()(const TexCoord &tcoord = TexCoord(0, 0), MatAttrMode mode = ATTR_MODE_COLOR) const;
	};

	enum {MAT_IOR, MAT_REFLECT, MAT_TRANSMIT, MAT_GLOSSINESS, MAT_LAST};
	
	class Material {
	protected:
		MatAttr *attr;

		virtual void SetDefaults();

	public:
		virtual ~Material();
		
		virtual void SetAttr(int attr_id, const MatAttr &attr);
		virtual const MatAttr &GetAttr(int attr_id) const;
		virtual MatAttr *GetAttrPtr(int attr_id) const;
	};
		
}

#endif	// _MATERIAL_HPP_
