#include "raytracer.hpp"
#include "object.hpp"
#include "animation.hpp"
#include "mat_phong.hpp"

using namespace xray;

// Abstract base class Object implementation

Object::Object() {
	mat = new MatPhong;
	manage_mat_ptr = true;
	two_sided = true;
}

Object::~Object() {
	if(manage_mat_ptr) {
		delete mat;
	}
}

void Object::SetTwoSided(bool enable) {
	two_sided = enable;
}

bool Object::GetTwoSided() const {
	return two_sided;
}

void Object::SetMaterial(Material *mat) {
	manage_mat_ptr = false;
	delete this->mat;
	this->mat = mat;
}

Material *Object::GetMaterial() const {
	return mat;
}


// QuadrObject (Object with Quadratic Surface) implementation

QuadrObject::QuadrObject() {
	surface = 0;
}

QuadrObject::~QuadrObject() {
	if(surface) delete surface;
}

void QuadrObject::SetSurface(void *srf) {
	surface = (Quadratic*)srf;
}

void *QuadrObject::GetSurface() {
	return surface;
}

TexCoord QuadrObject::GetTexCoords(const Vector3 &pt) const {
	Animation anim = GetActiveXRayContext()->scene->GetAnimation();
	Vector2 tcvec = surface->InvMap(pt, GetPRS(anim.current_time).rotation);

	return TexCoord(tcvec.x, tcvec.y);
}

bool QuadrObject::CheckIntersection(const Ray &ray) const {
	return FindIntersection(ray, 0);
}

bool QuadrObject::FindIntersection(const Ray &ray, SurfPoint *isect) const {
	if(surface->FindIntersection(ray, isect)) {
		if(two_sided || DotProduct(isect->normal, ray.dir) < 0.0) {
			return true;
		}
	}
	return false;
}
