#ifndef _OBJECT_HPP_
#define _OBJECT_HPP_

#include <string>
#include "n3dmath2/n3dmath2.hpp"
#include "gfx/3dgeom.hpp"
#include "material.hpp"

namespace xray {

	class Object : public XFormNode {
	protected:
		Material *mat;
		bool two_sided;
		bool manage_mat_ptr;
		
	public:
		Object();
		virtual ~Object();

		virtual void SetTwoSided(bool enable);
		virtual bool GetTwoSided() const;

		virtual void SetMaterial(Material *mat);
		virtual Material *GetMaterial() const;

		virtual void SetSurface(void *srf) = 0;
		virtual void *GetSurface() = 0;

		virtual TexCoord GetTexCoords(const Vector3 &pt) const = 0;

		virtual bool CheckIntersection(const Ray &ray) const = 0;
		virtual bool FindIntersection(const Ray &ray, SurfPoint *isect) const = 0;
	};

	class QuadrObject : public Object {
	protected:
		Quadratic *surface;

	public:
		QuadrObject();
		virtual ~QuadrObject();

		virtual void SetSurface(void *srf);
		virtual void *GetSurface();

		virtual TexCoord GetTexCoords(const Vector3 &pt) const ;

		virtual bool CheckIntersection(const Ray &ray) const;
		virtual bool FindIntersection(const Ray &ray, SurfPoint *isect) const;
	};

	/*
	class MeshObject : public Object {
	protected:
		TriMesh *surface;

	public:
		MeshObject();
		~MeshObject();
		
		virtual void SetSurface(void *srf);
		virtual void *GetSurface();

		virtual TexCoord GetTexCoords(const Vector3 &pt) const;
		
		virtual bool CheckIntersection(const Ray &ray) const;
		virtual bool FindIntersection(const Ray &ray, SurfPoint *isect) const;
	};
	*/

}
		
		

#endif	// _OBJECT_HPP_
