/*
Copyright 2004 John Tsiombikas <nuclear@siggraph.org>

This file is part of the X-Ray ray-tracing library.

X-Ray is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

X-Ray is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with X-Ray; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef _RAYTRACER_HPP_
#define _RAYTRACER_HPP_

#include "scene.hpp"
#include "n3dmath2/n3dmath2.hpp"
#include "gfx/pbuffer.hpp"
#include "gfx/color.hpp"

namespace xray {

	struct RenderStats {
		unsigned long primary_rays;
		unsigned long secondary_rays;
		unsigned long gloss_rays;
		unsigned long shadow_rays;
	};

	struct XRayContext {
		int x, y;
		PixelBuffer *pbuf;
		Vector3 *pix_pos;
		Vector3 eye_pos;
		
		int rays_per_pixel;
		int max_ray_depth;
		scalar_t ray_energy_threshold;
		int area_light_samples;
		int glossy_samples;

		// monte carlo stuff
		int max_diffuse_bounces;

		RenderStats stats;
		
		Scene *scene;
		void (*ProgressCallback)(float);
	};
	
	XRayContext *Init(int x, int y);
	void Destroy(XRayContext *xrc = 0);

	void SetActiveXRayContext(XRayContext *xrc);
	XRayContext *GetActiveXRayContext();

	void SetProgressCallback(void (*ProgressCallback)(float));

	const PixelBuffer *GetPixelBuffer();
	void SetActiveScene(Scene *scene);
	Scene *GetActiveScene();

	void ResetRenderStats();
	RenderStats *GetRenderStats();

	Color ShootRay(const Ray &ray, int ray_depth);
	scalar_t ShadowRay(const Vector3 &pt, const Light *light);
	Color SampleSolidAngle(const Ray &ray, scalar_t angle, int samples, int ray_depth);

	Color Shade(const Ray &ray, SurfPoint pt, const Object *obj, int ray_depth);
	Color ShadeMonteCarlo(const Ray &ray, SurfPoint pt, const Object *obj, int ray_depth);

	scalar_t Fresnel(scalar_t cos_theta, scalar_t ior);
}

#endif	// _RAYTRACER_HPP_
